#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from datetime import date

# import interfaces
from onf_website.reference.orga.model.interfaces import IStructureModel, IStructure
from zope.schema.interfaces import ITitledTokenizedTerm

# import packages
from pyams_alchemy import Base
from pyams_alchemy.engine import get_user_session
from pyams_alchemy.mixin import DynamicSchemaMixin
from sqlalchemy import Column, Integer, Unicode, Date, ForeignKey
from sqlalchemy.sql import and_, or_
from zope.interface import provider, implementer


PARENT_SCHEMA = 'refstruct'
PARENT_SESSION = 'REFSTRUCT'


@provider(IStructureModel)
@implementer(IStructure, ITitledTokenizedTerm)
class Structure(DynamicSchemaMixin, Base):
    """Base ONF structure"""

    __tablename__ = 'vm_bu_structures'
    __schema__ = PARENT_SCHEMA

    id = Column(Integer, primary_key=True)
    code = Column(Unicode(6))
    code_sign = Column(Unicode(6))
    libelle = Column(Unicode(30))
    libelle_long = Column(Unicode(40))
    niveau = Column(Integer)
    libelle_type = Column(Unicode(20))
    libelle_type_long = Column('libelle_long_type', Unicode(40))
    date_debut = Column('d_debut', Date)
    date_fin = Column('d_fin', Date)
    date_etat = Column('etat', Date)
    #code_parent = Column('code_parent', Unicode(6))
    complement_adresse = Column('adresse_comp', Unicode(40))
    num_voie = Column(Integer)
    ext_voie = Column(Unicode(5))
    nature_voie = Column('nat_voie', Unicode(32))
    nom_commune = Column(Unicode(32))
    code_commune = Column(Unicode(5))
    code_postal = Column(Unicode(10))
    bureau_distrib = Column('bur_distrib', Unicode(32))
    pays = Column(Unicode(6))
    code_iso_pays = Column('pays_code_iso', Unicode(2))
    categorie = Column('code_categorie', Unicode(10))

    @property
    def value(self):
        return self.code

    @property
    def token(self):
        return str(self.code)

    @property
    def title(self):
        if self.code_sign:
            return u'%s - %s' % (self.code_sign, self.libelle_long)
        else:
            return self.libelle_long

    @classmethod
    def get_default_params(cls, reference_date=None):
        if reference_date is None:
            reference_date = date.today()
        return [Structure.date_debut <= reference_date,
                or_(Structure.date_fin == None,
                    Structure.date_fin >= reference_date)]

    @classmethod
    def get(cls, code, reference_date=None, session=PARENT_SESSION):
        if reference_date is None:
            reference_date = date.today()
        if isinstance(code, (list, tuple)):
            params = [Structure.code.in_(code), ]
        else:
            params = [Structure.code == code, ]
        params.extend(cls.get_default_params(reference_date))
        session = get_user_session(session)
        return session.query(Structure).filter(and_(*params))

    @classmethod
    def find(cls, query, reference_date=None, exact=False, session=PARENT_SESSION):
        if isinstance(query, dict):
            if 'query' in query:
                code = query.get('query', '').strip()
                label = query.get('query', '').strip().upper()
            else:
                code = query.get('code', '')
                if isinstance(code, str):
                    code = code.strip()
                label = query.get('label', '').strip().upper()
            type = query.get('type', '').strip()
            category = query.get('category', None)
            if isinstance(category, str):
                category = category.strip().split(';')
        else:
            code = query.strip().upper()
            label = query.strip().upper()
            type = None
            category = None
        if reference_date is None:
            reference_date = date.today()
        params = []
        code_params = []
        if code:
            if exact:
                if isinstance(code, (list, tuple)):
                    code_params.append(Structure.code_sign.in_(code))
                else:
                    code_params.append(Structure.code_sign == code)
            else:
                if isinstance(code, (list, tuple)):
                    code_params.append(or_(*[Structure.code_sign.like(c + '%') for c in code]))
                else:
                    code_params.append(Structure.code_sign.like(code + '%'))
        if label:
            if exact:
                code_params.append(or_(Structure.libelle == label,
                                       Structure.libelle_long == label))
            else:
                code_params.append(or_(Structure.libelle.like('%' + label + '%'),
                                       Structure.libelle_long.like('%' + label + '%')))
        if code_params:
            params.append(or_(*code_params))
        if type:
            params.append(Structure.libelle_type == type)
        if category:
            params.append(Structure.categorie.in_(category))
        params.extend(cls.get_default_params(reference_date))
        params = [and_(*params), ]
        session = get_user_session(session)
        return session.query(Structure).filter(and_(*params))

    def get_parents(self, session=PARENT_SESSION):
        """Get a structure and all it's parents"""
        session = get_user_session(session)
        connection = session.connection()
        if connection.engine.url.startswith('oracle://'):
            statement = "select struc.*, level " \
                        "from {schema}.{tablename} struc " \
                        "start with (code='{code}') and (d_fin is null or d_fin > sysdate) " \
                        "connect by nocycle (prior code_parent = code) and " \
                        "(d_fin is null or d_fin > sysdate) " \
                        "order by level"
        else:
            statement = "with recursive q as (" \
                        "select * " \
                        "    from {schema}.{tablename} " \
                        "    where code='{code}' and (d_fin is null or d_fin > now()) " \
                        "    union all " \
                        "    select struc.* " \
                        "    from {schema}.{tablename} struc " \
                        "    where (d_fin is null) or (d_fin > now()) " \
                        "    join q on q.code_parent = struc.code " \
                        ") select * from q order by niveau"
        return session.query(Structure) \
                      .from_statement(statement.format(schema=self.__table__.schema,
                                                       tablename=self.__tablename__,
                                                       code=self.code))

    def get_first_parent(self, category, session=PARENT_SESSION):
        """Get first parent matching given categories"""
        if isinstance(category, str):
            category = category.split(';')
        for parent in reversed(self.getParents(session)[0:-1]):
            if parent.categorie in category:
                return parent

    def get_childs(self, session=PARENT_SESSION, category=None, direct_only=False, reference_date=None):
        """Get a structure and all it's childs"""
        if category:
            if isinstance(category, str):
                category = category.strip().split(';')
        else:
            category = ''
        if reference_date is None:
            reference_date = date.today()
        session = get_user_session(session)
        structures = session.query(Structure)
        if direct_only:
            structures = structures.filter(and_(or_(Structure.code == self.code,
                                                    Structure.code_parent == self.code),
                                                and_(Structure.date_debut <= reference_date,
                                                     or_(Structure.date_fin == None,
                                                         Structure.date_fin >= reference_date))))
            if category:
                structures = structures.filter(Structure.categorie.in_(category))
            return structures.order_by(Structure.code_sign)
        else:
            connection = session.connection()
            if connection.engine.url.startswith('oracle://'):
                statement = "select struc.* " \
                            "from {schema}.{tablename} struc " \
                            "start with (code='{code}') and (d_fin is null or d_fin > sysdate) " \
                            "connect by nocycle (code_parent = prior code) and " \
                            "(d_fin is null or d_fin > sysdate) " \
                            "order siblings by code_sign, libelle_long"
            else:
                statement = "with recursive q as (" \
                            "select * " \
                            "    from {schema}.{tablename} " \
                            "    where code='{code}' and (d_fin is null or d_fin > now()) " \
                            "    union all " \
                            "    select struc.* " \
                            "    from {schema}.{tablename} struc " \
                            "    where (d_fin is null) or (d_fin > now()) " \
                            "    join q on q.code = struc.code_parent " \
                            ") select * from q order by code_sign, libelle_long"
            result = structures.from_statement(statement.format(schema=self.__table__.schema,
                                                                tablename=self.__tablename__,
                                                                code=self.code))
            if category:
                result = filter(lambda x: x.categorie in category, result)
            return result

Structure.code_parent = Column('code_parent', Unicode(6), ForeignKey(Structure.code))
